@@ -1,8 +1,4 @@
 import sys
-if not hasattr(sys, 'frozen'):
-    import wxversion
-    wxversion.ensureMinimal('2.8')
-
 import wx
 from numpy import exp, random, arange, outer
 from wxmplot import ImageFrame
