@@ -1,13 +1,7 @@
 #!/usr/bin/python
 #
-# simple example of MPlot
+# simple example of wxMPlot
 import sys
-if not hasattr(sys, 'frozen'):
-    try:
-        import wxversion
-        wxversion.ensureMinimal('2.8')
-    except:
-        pass
 import wx
 import numpy
 import wxmplot
