@@ -7,8 +7,11 @@ import sys
 import time
 import os
 import wx
+is_wxPhoenix = 'phoenix' in wx.PlatformInfo
+
 import matplotlib
 from matplotlib.widgets import Lasso
+from matplotlib import dates
 
 from .utils import Printer
 
@@ -23,15 +26,13 @@ class BasePanel(wx.Panel):
          bindings for keyboard short-cuts
     """
     def __init__(self, parent, messenger=None,
-                 show_config_popup=True,
+                 show_config_popup=True, zoom_callback=None,
                  output_title=None, **kws):
 
         wx.Panel.__init__(self, parent, -1, **kws)
 
-        self.is_macosx = False
-        if os.name == 'posix':
-            if os.uname()[0] == 'Darwin':
-                self.is_macosx = True
+        self.is_macosx = (os.name == 'posix' and
+                          sys.platform.lower().startswith('darw'))
 
         self.messenger = messenger
         if messenger is None:
@@ -47,13 +48,15 @@ class BasePanel(wx.Panel):
 
         self.mouse_uptime = time.time()
         self.user_limits = {}
-        self.zoom_lims = []    # x, y coords zoom levels
+        # self.zoom_lims = []    # x, y coords zoom levels
         self.zoom_ini  = None  # x, y coords for zoom-box
+        self.zoom_callback = zoom_callback
         self.rbbox = None
         self.zdc = None
         self.cursor_modes = {}
         self.cursor_mode = 'report'
         self.parent = parent
+        self.motion_sbar = None
         self.printer = Printer(self, title=output_title)
         self.add_cursor_mode('report', motion = self.report_motion,
                              leftdown = self.report_leftdown)
@@ -74,6 +77,13 @@ class BasePanel(wx.Panel):
                                 self.__onMouseButtonEvent)
         self.canvas.mpl_connect("key_press_event",
                                 self.__onKeyEvent)
+
+        if os.name == 'posix':
+            def swallow_mouse(*args):
+                pass
+            self.canvas.CaptureMouse = swallow_mouse
+            self.canvas.ReleaseeMouse = swallow_mouse
+
         self.BuildPopup()
 
     def BuildPopup(self):
@@ -105,19 +115,19 @@ class BasePanel(wx.Panel):
 
     def unzoom_all(self, event=None):
         """ zoom out full data range """
-        if len(self.zoom_lims) > 0:
-            self.zoom_lims = [self.zoom_lims[0]]
+        if len(self.conf.zoom_lims) > 0:
+            self.conf.zoom_lims = [self.conf.zoom_lims[0]]
         self.unzoom(event)
 
     def unzoom(self, event=None, set_bounds=True):
         """ zoom out 1 level, or to full data range """
         lims = None
-        if len(self.zoom_lims) > 1:
-            lims = self.zoom_lims.pop()
+        if len(self.conf.zoom_lims) > 1:
+            lims = self.conf.zoom_lims.pop()
         ax = self.axes
         # print 'base unzoom ', lims, set_bounds
         if lims is None: # auto scale
-            self.zoom_lims = [None]
+            self.conf.zoom_lims = [None]
             xmin, xmax, ymin, ymax = self.data_range
             ax.set_xlim((xmin, xmax), emit=True)
             ax.set_ylim((ymin, ymax), emit=True)
@@ -193,7 +203,7 @@ class BasePanel(wx.Panel):
                             defaultDir = os.getcwd(),
                             defaultFile=ofile,
                             wildcard=file_choices,
-                            style=wx.SAVE|wx.CHANGE_DIR)
+                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)
 
         if dlg.ShowModal() == wx.ID_OK:
             path = dlg.GetPath()
@@ -255,6 +265,9 @@ class BasePanel(wx.Panel):
             return
         self.write_message("%f, %f" % (event.xdata, event.ydata), panel=1)
 
+    def onExport(self, event=None):
+        pass
+
     def onLeftDown(self, event=None):
         """ left button down: report x,y coords, start zooming mode"""
         if event is None:
@@ -268,13 +281,14 @@ class BasePanel(wx.Panel):
             return
         self.cursor_mode_action('leftup', event=event)
         self.canvas.draw_idle()
+        self.canvas.draw()
         self.ForwardEvent(event=event.guiEvent)
 
     def ForwardEvent(self, event=None):
         """finish wx event, forward it to other wx objects"""
         if event is not None:
             event.Skip()
-            if os.name == 'posix' or  self.HasCapture():
+            if self.HasCapture():
                 try:
                     self.ReleaseMouse()
                 except:
@@ -309,19 +323,26 @@ class BasePanel(wx.Panel):
         """ formatter for date x-data. primitive, and probably needs
         improvement, following matplotlib's date methods.
         """
-        interval = self.axes.xaxis.get_view_interval()
-        ticks = self.axes.xaxis.get_major_locator()()
-        span = max(interval) - min(interval)
+
+        if x < 1: x = 1
+
+        span = self.axes.xaxis.get_view_interval()
+        tmin = max(1.0, span[0])
+        tmax = max(2.0, span[1])
+        tmin = time.mktime(dates.num2date(tmin).timetuple())
+        tmax = time.mktime(dates.num2date(tmax).timetuple())
+        nhours = (tmax - tmin)/3600.0
         fmt = "%m/%d"
-        if span < 1800:
-            fmt = "%I%p \n%M:%S"
-        elif span < 86400*5:
-            fmt = "%m/%d \n%H:%M"
-        elif span < 86400*20:
-            fmt = "%m/%d"
-        # print 'date formatter  span: ', span, fmt
-        s = time.strftime(fmt, time.localtime(x))
-        return s
+        if nhours < 0.1:
+            fmt = "%H:%M\n%Ssec"
+        elif nhours < 4:
+            fmt = "%m/%d\n%H:%M"
+        elif nhours < 24*8:
+            fmt = "%m/%d\n%H:%M"
+        try:
+            return time.strftime(fmt, dates.num2date(x).timetuple())
+        except:
+            return "?"
 
     def xformatter(self, x, pos):
         " x-axis formatter "
@@ -435,6 +456,8 @@ class BasePanel(wx.Panel):
         handle_event = handlers.get((button, event.name), None)
         if hasattr(handle_event, '__call__'):
             handle_event(event)
+        event.guiEvent.Skip()
+
 
     def __onMouseMotionEvent(self, event=None):
         """Draw a cursor over the axes"""
@@ -467,14 +490,16 @@ class BasePanel(wx.Panel):
         zdc.SetBrush(wx.TRANSPARENT_BRUSH)
         zdc.SetPen(wx.Pen('White', 2, wx.SOLID))
         zdc.ResetBoundingBox()
-        zdc.BeginDrawing()
+        if not is_wxPhoenix:
+            zdc.BeginDrawing()
 
         # erase previous box
         if self.rbbox is not None:
             zdc.DrawRectangle(*self.rbbox)
         self.rbbox = (x0, y0, width, height)
         zdc.DrawRectangle(*self.rbbox)
-        zdc.EndDrawing()
+        if not is_wxPhoenix:
+            zdc.EndDrawing()
 
     def zoom_leftdown(self, event=None):
         """leftdown event handler for zoom mode"""
@@ -503,8 +528,8 @@ class BasePanel(wx.Panel):
                 xmin, xmax = ax.get_xlim()
                 ymin, ymax = ax.get_ylim()
                 zlims[ax] = [xmin, xmax, ymin, ymax]
-            if len(self.zoom_lims) == 0:
-                self.zoom_lims.append(zlims)
+            if len(self.conf.zoom_lims) == 0:
+                self.conf.zoom_lims.append(zlims)
             # for multiple axes, we first collect all the new limits, and
             # only then apply them
             for ax in self.fig.get_axes():
@@ -520,17 +545,24 @@ class BasePanel(wx.Panel):
 
                 tlims[ax] = [min(x0, x1), max(x0, x1),
                              min(y0, y1), max(y0, y1)]
-            self.zoom_lims.append(tlims)
+            self.conf.zoom_lims.append(tlims)
             # now apply limits:
             self.set_viewlimits()
 
+            if callable(self.zoom_callback):
+                self.zoom_callback(wid=self.GetId(), limits=tlims[ax])
+
     def lasso_motion(self, event=None):
         """motion event handler for lasso mode"""
         self.report_motion(event=event)
 
     def lasso_leftdown(self, event=None):
         """leftdown event handler for lasso mode"""
-        self.report_leftdown(event=event)
+        try:
+            self.report_leftdown(event=event)
+        except:
+            return
+
         if event.inaxes:
             # set lasso color
             color='goldenrod'
@@ -562,7 +594,13 @@ class BasePanel(wx.Panel):
                 x, y = self.axes.transData.inverted().transform((event.x, event.y))
             except:
                 pass
-        self.write_message(fmt % (x, y), panel=1)
+        if self.motion_sbar is None:
+            try:
+                self.motion_sbar = self.nstatusbar-1
+            except AttributeError:
+                self.motion_sbar = 1
+
+        self.write_message(fmt % (x, y), panel=self.motion_sbar)
 
     def Print(self, event=None, **kw):
         self.printer.Print(event=event, **kw)
@@ -572,4 +610,3 @@ class BasePanel(wx.Panel):
 
     def PrintSetup(self, event=None, **kw):
         self.printer.Setup(event=event, **kw)
-
