@@ -2,26 +2,30 @@ import wx
 import matplotlib.cm as colormap
 from .colors import register_custom_colormaps
 
-ColorMap_List = ['gray']
 
 cm_names = register_custom_colormaps()
 
-for cm in cm_names:
-    ColorMap_List.append(cm)
+# for cm in cm_names:
+#     if cm not in cm_names:
+#         ColorMap_List.append(cm)
 
-for cm in ('coolwarm', 'cool', 'hot', 'jet', 'Reds', 'Greens',
-           'Blues', 'copper', 'spring', 'summer', 'autumn', 'winter',
-           'hsv', 'Spectral'):
-    # 'gist_earth', 'gist_yarg', 'gist_rainbow',
-    # 'gist_stern',
-    # 'PiYG', 'PRGn', 'Spectral', 'Accent', 'YlGn', 'YlGnBu',
-    # 'RdBu', 'RdPu', 'RdYlBu', 'RdYlGn', 'ocean', 'terrain'
-    if hasattr(colormap, cm):
+ColorMap_List = []
+
+for cm in ('gray', 'coolwarm', 'viridis', 'inferno', 'plasma', 'magma',
+           'red', 'green', 'blue', 'magenta', 'yellow', 'cyan', 'Reds',
+           'Greens', 'Blues', 'cool', 'hot', 'copper', 'red_heat',
+           'green_heat', 'blue_heat', 'spring', 'summer', 'autumn',
+           'winter', 'ocean', 'terrain', 'jet', 'stdgamma', 'hsv',
+           'Accent', 'Spectral', 'PiYG', 'PRGn', 'Spectral', 'YlGn', 'YlGnBu',
+           'RdBu', 'RdPu', 'RdYlBu', 'RdYlGn'):
+    if cm in cm_names or hasattr(colormap, cm):
         ColorMap_List.append(cm)
 
-Interp_List = ('nearest', 'bilinear', 'bicubic', 'gaussian', 'catrom',
-               'spline16', 'spline36', 'hanning', 'hamming', 'hermite',
-               'kaiser', 'quadric', 'bessel', 'mitchell', 'sinc',
+
+
+Interp_List = ('nearest', 'bilinear', 'bicubic', 'quadric', 'gaussian',
+               'catrom', 'spline16', 'spline36', 'hanning', 'hamming',
+               'hermite', 'kaiser', 'bessel', 'mitchell', 'sinc',
                'lanczos')
 
 class ImageConfig:
@@ -29,22 +33,23 @@ class ImageConfig:
         self.axes   = axes
         self.fig  = fig
         self.canvas  = canvas
-        self.cmap  = {'int': colormap.gray}
+        self.cmap  = [colormap.gray, colormap.gray, colormap.gray]
         self.cmap_reverse = False
         self.interp = 'nearest'
         self.log_scale = False
         self.flip_ud = False
         self.flip_lr = False
         self.rot  = False
+        self.auto_contrast = False
+        self.auto_contrast_level = 1.0
         self.datalimits = [None, None, None, None]
-        self.cmap_lo = {'int': 0}
-        self.cmap_hi = {'int': 100}
-        self.cmap_range = 100
-        self.tricolor_bg = 'black'        
-        self.cmap_splines = {'int': None, 'red': None, 'green': None, 'blue': None}
-        self.auto_intensity = False
-        self.int_lo = {'int': '0', 'red': '0', 'green': '0', 'blue': '0'}
-        self.int_hi = {'int': '1', 'red': '1', 'green': '1', 'blue': '1'}
+        self.cmap_lo = [0, 0, 0]
+        self.cmap_range = 1000
+        self.cmap_hi = [1000, 1000, 1000]
+        self.tricolor_bg = 'black'
+        self.tricolor_mode = 'rgb'
+        self.int_lo = [0, 0, 0]
+        self.int_hi = [1, 1, 1]
         self.data = None
         self.indices = None
         self.title = 'image'
@@ -56,6 +61,7 @@ class ImageConfig:
         self.cursor_mode = 'zoom'
         self.zoombrush = wx.Brush('#040410',  wx.SOLID)
         self.zoompen   = wx.Pen('#101090',  3, wx.SOLID)
+        self.zoom_lims = []
 
     def relabel(self):
         " re draw labels (title, x,y labels)"
@@ -75,7 +81,7 @@ class ImageConfig:
         the new intensity to go
         from (0, 0.5, 0.5) (for Red=0)  to (0, 0, 0) (for Red=1)
         and so on for the Green and Blue maps.
-        
+
         Thus the image will be transformed from
           old intensity                new intensity
           (0.0, 0.0, 0.0) (black)   (1.0, 1.0, 1.0) (white)
@@ -91,5 +97,14 @@ class ImageConfig:
         out[:,:,1] = tmp[:,:,0] + tmp[:,:,2]
         out[:,:,2] = tmp[:,:,0] + tmp[:,:,1]
         return out
-        
 
+    def rgb2cmy(self, img, whitebg=False):
+        """transforms image from RGB to CMY"""
+        tmp = img*1.0
+        if whitebg:
+            tmp = (1.0 - (img - img.min())/(img.max() - img.min()))
+        out = tmp*0.0
+        out[:,:,0] = (tmp[:,:,1] + tmp[:,:,2])/2.0
+        out[:,:,1] = (tmp[:,:,0] + tmp[:,:,2])/2.0
+        out[:,:,2] = (tmp[:,:,0] + tmp[:,:,1])/2.0
+        return out
