@@ -33,21 +33,24 @@ class ImagePanel(BasePanel):
 
     def __init__(self, parent, messenger=None, data_callback=None,
                  cursor_callback=None, lasso_callback=None,
-                 contour_callback=None,
-                 size=(525, 450), dpi=100,
+                 redraw_callback=None, zoom_callback=None,
+                 contour_callback=None, size=(525, 450), dpi=100,
                  output_title='Image', **kws):
+
         matplotlib.rc('lines', linewidth=2)
         BasePanel.__init__(self, parent,
                            output_title=output_title,
-                           messenger=messenger, **kws)
+                           messenger=messenger,
+                           zoom_callback=zoom_callback, **kws)
+
         self.conf = ImageConfig()
         self.conf.title = output_title
         self.cursor_mode = 'zoom'
-
         self.data_callback = data_callback
         self.cursor_callback = cursor_callback
         self.lasso_callback = lasso_callback
         self.contour_callback = contour_callback
+        self.redraw_callback = redraw_callback
 
         self.win_config = None
         self.data_shape = None
@@ -62,7 +65,8 @@ class ImagePanel(BasePanel):
 
     def display(self, data, x=None, y=None, xlabel=None, ylabel=None,
                 style=None, nlevels=None, levels=None, contour_labels=None,
-                store_data=True,  col='int', **kws):
+                store_data=True, col=0, unzoom=True, auto_contrast=False,
+                **kws):
         """
         generic display, using imshow (default) or contour
         """
@@ -74,8 +78,11 @@ class ImagePanel(BasePanel):
         conf.log_scale = False
         conf.rot, conf.flip_ud, conf.flip_lr = False, False, False
         conf.highlight_areas = []
+        if 1 in data.shape:
+            data = data.squeeze()
         self.data_shape = data.shape
         self.data_range = [0, data.shape[1], 0, data.shape[0]]
+        conf.auto_contrast = auto_contrast
 
         if x is not None:
             self.xdata = np.array(x)
@@ -85,6 +92,7 @@ class ImagePanel(BasePanel):
             self.ydata = np.array(y)
             if self.ydata.shape[0] != data.shape[0]:
                 self.ydata = None
+
         if xlabel is not None:
             self.xlab = xlabel
         if ylabel is not None:
@@ -128,12 +136,16 @@ class ImagePanel(BasePanel):
             if hasattr(self.contour_callback , '__call__'):
                 self.contour_callback(levels=clevels)
         else: # image
-            img = (data -data.min()) /(1.0*data.max() - data.min())
+            if data.max() == data.min():
+                img = data
+            else:
+                img = (data - data.min()) /(1.0*data.max() - data.min())
             self.conf.image = self.axes.imshow(img, cmap=self.conf.cmap[col],
                                                interpolation=self.conf.interp)
 
         self.axes.set_axis_off()
-        self.unzoom_all()
+        if unzoom:
+            self.unzoom_all()
         if hasattr(self.data_callback, '__call__'):
             self.data_callback(data, x=x, y=y, **kws)
 
@@ -142,7 +154,7 @@ class ImagePanel(BasePanel):
         self.indices_thread.start()
 
 
-    def add_highlight_area(self, mask, label=None, col='int'):
+    def add_highlight_area(self, mask, label=None, col=0):
         """add a highlighted area -- outline an arbitrarily shape --
         as if drawn from a Lasso event.
 
@@ -151,7 +163,7 @@ class ImagePanel(BasePanel):
         """
         patch = mask * np.ones(mask.shape) * 0.9
         cmap = self.conf.cmap[col]
-        area = self.axes.contour(patch, cmap=cmap, levels=[0.8])
+        area = self.axes.contour(patch, cmap=cmap, levels=[0, 1])
         self.conf.highlight_areas.append(area)
         col = None
         if hasattr(cmap, '_lut'):
@@ -159,8 +171,9 @@ class ImagePanel(BasePanel):
             col  = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
 
         if label is not None:
-            fmt = {0.8: label, 0.9: label}
-            self.axes.clabel(area, fontsize=9, inline=1, fmt=fmt, colors=col)
+            def fmt(*args, **kws): return label
+            self.axes.clabel(area, fontsize=9, fmt=fmt,
+                             colors=col, rightside_up=True)
 
         if col is not None:
             for l in area.collections:
@@ -168,26 +181,26 @@ class ImagePanel(BasePanel):
 
         self.canvas.draw()
 
-    def set_viewlimits(self, axes=None, autoscale=False):
+    def set_viewlimits(self, axes=None):
         """ update xy limits of a plot"""
         if axes is None:
             axes = self.axes
 
         xmin, xmax, ymin, ymax = self.data_range
-        if not autoscale and len(self.zoom_lims) >1:
-            zlims = self.zoom_lims[-1]
+        if len(self.conf.zoom_lims) >1:
+            zlims = self.conf.zoom_lims[-1]
             if axes in zlims:
                 xmin, xmax, ymin, ymax = zlims[axes]
 
-        xmin = int(max(self.data_range[0], xmin) + 0.5)
-        xmax = int(min(self.data_range[1], xmax) + 0.5)
-        ymin = int(max(self.data_range[2], ymin) + 0.5)
-        ymax = int(min(self.data_range[3], ymax) + 0.5)
+        xmin = max(self.data_range[0], xmin)
+        xmax = min(self.data_range[1], xmax)
+        ymin = max(self.data_range[2], ymin)
+        ymax = min(self.data_range[3], ymax)
         if (xmax < self.data_range[0] or
             xmin > self.data_range[1] or
             ymax < self.data_range[2] or
             ymin > self.data_range[3] ):
-            self.zoom_lims.pop()
+            self.conf.zoom_lims.pop()
             return
 
         if abs(xmax-xmin) < 2:
@@ -198,17 +211,12 @@ class ImagePanel(BasePanel):
             ymin = int(0.5*(ymax+xmin) - 1)
             ymax = ymin + 2
 
-        self.axes.set_xlim((xmin,xmax),emit=True)
-        self.axes.set_ylim((ymin,ymax),emit=True)
-        self.axes.update_datalim(((xmin,ymin),(xmax,ymax)))
-        if autoscale:
-            self.axes.set_xbound(self.axes.xaxis.get_major_locator().view_limits(xmin,xmax))
-            self.axes.set_ybound(self.axes.yaxis.get_major_locator().view_limits(ymin,ymax))
+        self.axes.set_xlim((xmin, xmax),emit=True)
+        self.axes.set_ylim((ymin, ymax),emit=True)
+        self.axes.update_datalim(((xmin, ymin), (xmax, ymax)))
+
         self.conf.datalimits = [xmin, xmax, ymin, ymax]
-        try:
-            self.redraw()
-        except ValueError:
-            pass
+        self.redraw()
 
     def clear(self):
         """ clear plot """
@@ -227,11 +235,11 @@ class ImagePanel(BasePanel):
 
         self.canvas = FigureCanvasWxAgg(self, -1, self.fig)
         self.fig.set_facecolor('#FFFFFD')
-        
+
         self.conf.axes  = self.axes
         self.conf.fig   = self.fig
         self.conf.canvas= self.canvas
-        
+
         # self.canvas.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
         # This way of adding to sizer allows resizing
         sizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -276,7 +284,7 @@ class ImagePanel(BasePanel):
     ####
     ## GUI events, overriding BasePanel components
     ####
-    def exportASCII(self, event=None):
+    def onExport(self, event=None):
         ofile  = ''
         title = 'unknown map'
         if self.conf.title is not None:
@@ -294,11 +302,12 @@ class ImagePanel(BasePanel):
 
         ofile = ofile + '.dat'
         orig_dir = os.path.abspath(os.curdir)
-
+        file_choices = "DAT (*.dat)|*.dat|ALL FILES (*.*)|*.*"
         dlg = wx.FileDialog(self, message='Export Map Data to ASCII...',
                             defaultDir = os.getcwd(),
                             defaultFile=ofile,
-                            style=wx.SAVE|wx.CHANGE_DIR)
+                            wildcard=file_choices,
+                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)
 
         if dlg.ShowModal() == wx.ID_OK:
             self.writeASCIIFile(dlg.GetPath(), title=title)
@@ -348,22 +357,68 @@ class ImagePanel(BasePanel):
     def unzoom(self, event=None, set_bounds=True):
         """ zoom out 1 level, or to full data range """
         lims = None
-        if len(self.zoom_lims) > 1:
-            lims = self.zoom_lims.pop()
+        if len(self.conf.zoom_lims) > 1:
+            lims = self.conf.zoom_lims.pop()
         ax = self.axes
         if lims is None: # auto scale
-            self.zoom_lims = [None]
+            self.conf.zoom_lims = [None]
             xmin, xmax, ymin, ymax = self.data_range
             lims = {self.axes: [xmin, xmax, ymin, ymax]}
         self.set_viewlimits()
         self.canvas.draw()
 
+    def zoom_leftup(self, event=None):
+        """leftup event handler for zoom mode  in images"""
+        if self.zoom_ini is None:
+            return
+
+        ini_x, ini_y, ini_xd, ini_yd = self.zoom_ini
+        try:
+            dx = abs(ini_x - event.x)
+            dy = abs(ini_y - event.y)
+        except:
+            dx, dy = 0, 0
+        t0 = time.time()
+        self.rbbox = None
+        self.zoom_ini = None
+        if (dx > 3) and (dy > 3) and (t0-self.mouse_uptime)>0.1:
+            self.mouse_uptime = t0
+            zlims, tlims = {}, {}
+            ax =  self.axes
+            xmin, xmax = ax.get_xlim()
+            ymin, ymax = ax.get_ylim()
+
+            zlims[ax] = [xmin, xmax, ymin, ymax]
+
+            if len(self.conf.zoom_lims) == 0:
+                self.conf.zoom_lims.append(zlims)
+
+
+            ax_inv = ax.transData.inverted
+            try:
+                x1, y1 = ax_inv().transform((event.x, event.y))
+            except:
+                x1, y1 = self.x_lastmove, self.y_lastmove
+            try:
+                x0, y0 = ax_inv().transform((ini_x, ini_y))
+            except:
+                x0, y0 = ini_xd, ini_yd
+
+            tlims[ax] = [int(round(min(x0, x1))), int(round(max(x0, x1))),
+                         int(round(min(y0, y1))), int(round(max(y0, y1)))]
+            self.conf.zoom_lims.append(tlims)
+            # now apply limits:
+            self.set_viewlimits()
+            if callable(self.zoom_callback):
+                self.zoom_callback(wid=self.GetId(), limits=tlims[ax])
+
+
     def unzoom_all(self, event=None):
         """ zoom out full data range """
-        self.zoom_lims = [None]
+        self.conf.zoom_lims = [None]
         self.unzoom(event)
 
-    def redraw(self, col='int'):
+    def redraw(self, col=0):
         """redraw image, applying the following:
         rotation, flips, log scale
         max/min values from sliders or explicit intensity ranges
@@ -376,6 +431,9 @@ class ImagePanel(BasePanel):
         if conf.rot:
             if self.xdata is not None:
                 self.xdata = self.xdata[::-1]
+            if self.ydata is not None:
+                self.ydata = self.ydata[:]
+
             self.display(np.rot90(conf.data),
                          x=self.ydata, xlabel=self.ylab,
                          y=self.xdata, ylabel=self.xlab)
@@ -383,7 +441,7 @@ class ImagePanel(BasePanel):
         img = conf.data
         if img is None: return
         if len(img.shape) == 2:
-            col = 'int'
+            col = 0
         if self.conf.style == 'image':
             if conf.flip_ud:   img = np.flipud(img)
             if conf.flip_lr:   img = np.fliplr(img)
@@ -391,36 +449,37 @@ class ImagePanel(BasePanel):
                 img = np.log10(1 + 9.0*img)
 
         # apply intensity scale for current limited (zoomed) image
-        imin = float(conf.int_lo[col])
-        imax = float(conf.int_hi[col])
-        if conf.log_scale:
-            imin = np.log10(1 + 9.0*imin)
-            imax = np.log10(1 + 9.0*imax)
-        if conf.auto_intensity:
+        if len(img.shape) == 2:
+            # apply clipped color scale, as from sliders
+
+            imin = float(conf.int_lo[col])
+            imax = float(conf.int_hi[col])
+            if conf.log_scale:
+                imin = np.log10(1 + 9.0*imin)
+                imax = np.log10(1 + 9.0*imax)
+
             (xmin, xmax, ymin, ymax) = self.conf.datalimits
             if xmin is None:  xmin = 0
             if xmax is None:  xmax = img.shape[1]
             if ymin is None:  ymin = 0
             if ymax is None:  ymax = img.shape[0]
-            imin = np.min(img[ymin:ymax, xmin:xmax])
-            imax = np.max(img[ymin:ymax, xmin:xmax])
-        # apply clipped color scale, as from sliders
-        if len(img.shape) == 2:
+
+
             img = (img - imin)/(imax - imin + 1.e-8)
-            mlo = conf.cmap_lo[col]/(1.0*conf.cmap_range)
-            mhi = conf.cmap_hi[col]/(1.0*conf.cmap_range)
+            mlo = conf.cmap_lo[0]/(1.0*conf.cmap_range)
+            mhi = conf.cmap_hi[0]/(1.0*conf.cmap_range)
             if self.conf.style == 'image':
                 conf.image.set_data(np.clip((img - mlo)/(mhi - mlo + 1.e-8), 0, 1))
                 conf.image.set_interpolation(conf.interp)
         else:
             r, g, b = img[:,:,0], img[:,:,1], img[:,:,2]
 
-            rmin = float(conf.int_lo['red'])
-            rmax = float(conf.int_hi['red'])
-            gmin = float(conf.int_lo['green'])
-            gmax = float(conf.int_hi['green'])
-            bmin = float(conf.int_lo['blue'])
-            bmax = float(conf.int_hi['blue'])
+            rmin = float(conf.int_lo[0])
+            rmax = float(conf.int_hi[0])
+            gmin = float(conf.int_lo[1])
+            gmax = float(conf.int_hi[1])
+            bmin = float(conf.int_lo[2])
+            bmax = float(conf.int_hi[2])
             if conf.log_scale:
                 rmin = np.log10(1 + 9.0*rmin)
                 rmax = np.log10(1 + 9.0*rmax)
@@ -429,12 +488,12 @@ class ImagePanel(BasePanel):
                 bmin = np.log10(1 + 9.0*bmin)
                 bmax = np.log10(1 + 9.0*bmax)
 
-            rlo = conf.cmap_lo['red']/(1.0*conf.cmap_range)
-            rhi = conf.cmap_hi['red']/(1.0*conf.cmap_range)
-            glo = conf.cmap_lo['green']/(1.0*conf.cmap_range)
-            ghi = conf.cmap_hi['green']/(1.0*conf.cmap_range)
-            blo = conf.cmap_lo['blue']/(1.0*conf.cmap_range)
-            bhi = conf.cmap_hi['blue']/(1.0*conf.cmap_range)
+            rlo = conf.cmap_lo[0]/(1.0*conf.cmap_range)
+            rhi = conf.cmap_hi[0]/(1.0*conf.cmap_range)
+            glo = conf.cmap_lo[1]/(1.0*conf.cmap_range)
+            ghi = conf.cmap_hi[1]/(1.0*conf.cmap_range)
+            blo = conf.cmap_lo[2]/(1.0*conf.cmap_range)
+            bhi = conf.cmap_hi[2]/(1.0*conf.cmap_range)
             r = (r - rmin)/(rmax - rmin + 1.e-8)
             g = (g - gmin)/(gmax - gmin + 1.e-8)
             b = (b - bmin)/(bmax - bmin + 1.e-8)
@@ -443,13 +502,19 @@ class ImagePanel(BasePanel):
             inew[:,:,0] = np.clip((r - rlo)/(rhi - rlo + 1.e-8), 0, 1)
             inew[:,:,1] = np.clip((g - glo)/(ghi - glo + 1.e-8), 0, 1)
             inew[:,:,2] = np.clip((b - blo)/(bhi - blo + 1.e-8), 0, 1)
-            if conf.tricolor_bg.startswith('wh'):
+
+            whitebg = conf.tricolor_bg.startswith('wh')
+
+            if whitebg:
                 inew = conf.tricolor_white_bg(inew)
-              
+
             if self.conf.style == 'image':
                 conf.image.set_data(inew)
                 conf.image.set_interpolation(conf.interp)
         self.canvas.draw()
+        if callable(self.redraw_callback):
+            self.redraw_callback(wid=self.GetId())
+
 
     def report_leftdown(self,event=None):
         if event == None:
@@ -457,7 +522,7 @@ class ImagePanel(BasePanel):
         if event.xdata is None or event.ydata is None:
             return
 
-        ix, iy = round(event.xdata), round(event.ydata)
+        ix, iy = int(round(event.xdata)), int(round(event.ydata))
         if self.conf.flip_ud:  iy = self.conf.data.shape[0] - iy
         if self.conf.flip_lr:  ix = self.conf.data.shape[1] - ix
 
